/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/
#include <stdlib.h>
#include "ER-EPD0583-1.h"
#include "Debug.h"
#include "GUI_Paint.h"

/******************************************************************************
function :	Software reset
parameter:
******************************************************************************/
void EPD5IN83::EPD_Reset(void)
{
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
    EPD_RST_0;
    Dev->Dev_Delay_ms(200);
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
}

/******************************************************************************
function :	send command
parameter:
    Reg : Command register
******************************************************************************/
void EPD5IN83::EPD_SendCommand(UBYTE Reg)
{
    EPD_DC_0;
    EPD_CS_0;
    SPI_Write_Byte(Reg);
    EPD_CS_1;
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
void EPD5IN83::EPD_SendData(UBYTE Data)
{
    EPD_DC_1;
    EPD_CS_0;
    SPI_Write_Byte(Data);
    EPD_CS_1;
}

/******************************************************************************
function :	Wait until the busy_pin goes LOW
parameter:
******************************************************************************/
void EPD5IN83::EPD_WaitUntilIdle(void)
{
    while(EPD_BUSY_RD == 0) {      //HIGH: idle, LOW: busy
        Dev->Dev_Delay_ms(100);
    }
}

/******************************************************************************
function :	Turn On Display
parameter:
******************************************************************************/
void EPD5IN83::EPD_TurnOnDisplay(void)
{
    EPD_SendCommand(DISPLAY_REFRESH);
    Dev->Dev_Delay_ms(100);
    EPD_WaitUntilIdle();
}

/******************************************************************************
function :	Initialize the e-Paper register
parameter:
******************************************************************************/
UBYTE EPD5IN83::EPD_Init(void)
{
    Dev->System_Init();

    EPD_Reset();


   //Fiti Command
        EPD_SendCommand(0xF8);  // Enter FITI Command
        EPD_SendData(0x60); 
        EPD_SendData(0xA5); 
               
        EPD_SendCommand(0xF8);  // Enter FITI Command
        EPD_SendData(0x89); 
        EPD_SendData(0xA5);      
        
        EPD_SendCommand(0xF8);  // Sorting off
        EPD_SendData(0xA1);
        EPD_SendData(0x00);
        
        EPD_SendCommand(0xF8);  //Vcom driving
        EPD_SendData(0x73);
        EPD_SendData(0x07);
        
        EPD_SendCommand(0xF8);  
        EPD_SendData(0x76);
        EPD_SendData(0x1F);      
        
        EPD_SendCommand(0xF8);   //boost constant on time
        EPD_SendData(0x7E);
        EPD_SendData(0x31); 
        
        EPD_SendCommand(0xF8);  
        EPD_SendData(0xB8);  
        EPD_SendData(0x80);  
        
        EPD_SendCommand(0xF8);   //vgl=> GND:08, HZ:00[default]
        EPD_SendData(0x92);  
        EPD_SendData(0x00);  
        
        EPD_SendCommand(0xF8);   //VCOM(2frme off)=> GND:01(0x88=06)[default], HZ:11  
        EPD_SendData(0x87);  
        EPD_SendData(0x01);
        
        EPD_SendCommand(0xF8);   //r_vcom_init_sel
        EPD_SendData(0x88);  
        EPD_SendData(0x06);        
        
        EPD_SendCommand(0xF8);   
        EPD_SendData(0xA8);  
        EPD_SendData(0x30);
        
    EPD_SendCommand(POWER_ON);
    EPD_WaitUntilIdle();

    EPD_SendCommand(PANEL_SETTING);
    EPD_SendData(0x4F);
    
    EPD_SendCommand(PLL_CONTROL );
    EPD_SendData(0x3C); //set 50hz   

    EPD_SendCommand(RESOLUTION_SETTING);
    EPD_SendData(0x02);
    EPD_SendData(0x88);
    EPD_SendData(0x01);
    EPD_SendData(0xE0);

    return 0;
}

/******************************************************************************
function :	Clear screen
parameter:
******************************************************************************/
void EPD5IN83::EPD_Clear(void)
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;

    EPD_SendCommand(DATA_START_TRANSMISSION_1);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0xFF);
        }
    }
    EPD_SendCommand(DATA_START_TRANSMISSION_2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0x00);
        }
    }
}

/******************************************************************************
function :	Sends the image buffer in RAM to e-Paper and displays
parameter:
******************************************************************************/
void EPD5IN83::EPD_Display(void)
{
    UDOUBLE Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    EPD_SendCommand(DATA_START_TRANSMISSION_1);
    for (UDOUBLE j = 0; j < Height; j++) {
        for (UDOUBLE i = 0; i < Width; i++) {
            EPD_SendData(spiram->SPIRAM_RD_Byte(i + j * Width));
        }
    }
    Dev->Dev_Delay_ms(2);
    
    EPD_SendCommand(DATA_START_TRANSMISSION_2);
    for (UDOUBLE j = 0; j < Height; j++) {
        for (UDOUBLE i = 0; i < Width; i++) {
            EPD_SendData(spiram->SPIRAM_RD_Byte(i + j * Width + Paint_Image.Image_Offset));
        }
    }
    Dev->Dev_Delay_ms(2);
    EPD_TurnOnDisplay();
}

/******************************************************************************
function :	Enter sleep mode
parameter:
******************************************************************************/
void EPD5IN83::EPD_Sleep(void)
{
    EPD_SendCommand(POWER_OFF);
    EPD_WaitUntilIdle();
    EPD_SendCommand(DEEP_SLEEP);
    EPD_SendData(0XA5);
}
